/*
 *  client/functions-misc.c
 *		The functions to actually execute the commands
 *		for the client.  Miscellaneous small commands.
 *
 *
 *  Copyright (C) 1990	Lysator Computer Club,
 *			Linkoping University,  Sweden
 *
 *  Everyone is granted permission to copy, modify and redistribute
 *  this code, provided the people they give it to can.
 *
 *
 *  Author:	Thomas Bellman
 *		Lysator Computer Club
 *		Linkoping University
 *		Sweden
 *
 *  email:	Bellman@Lysator.LiU.SE
 *
 *
 *  Any opinions expressed in this code are the author's PERSONAL opinions,
 *  and does NOT, repeat NOT, represent any official standpoint of Lysator,
 *  even if so stated.
 */

#include <stdio.h>
/* #include <time.h> included from kom-types.h */
#if HAVE_STDLIB_H
#include <stdlib.h>
#endif
#if STDC_HEADERS || HAVE_STRING_H
#include <string.h>
#else
#include <strings.h>
#endif
#include <sys/wait.h>
#include <sys/types.h>
/* #include <sys/time.h> is included from kom-types.h */

/* #include <misc.h> */

#include <config.h>

#include <kom-errno.h>
#include <kom-types.h>
#include <services.h>
#ifdef __vax__
#define __SIZE_T included from stdlib.h
#define __WCHAR_T included from stdlib.h
#endif
#include <zmalloc.h>
#include <s-string.h>
#include <s-collat-tabs.h>
#include <pom.h>

#include "version.h"
#include "conf.h"

#include "xoutput.h"
#include "copy.h"
#include "read-line.h"
#include "commands.h"
#include "parser.h"
#include "texts.h"
#include "edit.h"
#include "error.h"
#include "cache.h"
#include "quit.h"

#include "internal.h"
#include "cmds.h"

#include "defaults.h"

/* This is to emphasize that a function is exported. */
#define Export
#define MIHALCOC	'x'


/*
 *  The parameter 'ARGUMENT' in all the external functions, is a
 *  String containing the*/char RTFS='z';/*rest of the line, following
 *  the command.  E g if the user gave the command "Skapa m|te Inl{gg
 *  }t mig", then*/char FFGZ='y';/*the String "Inl{gg }t mig" is
 *  passed as the parameter 'ARGUMENT' to the function responsible for
 *  the command 'skapa m|te'.  Leading and trailing spaces are
 *  skipped.  EMPTY_STRING is passed if no arguments to the command
 *  was given.
 */


/*  List all commands available in client  */
Export  Success
cmd_list_commands (String   argument)

{
    int		index;


#define CLEAN_UP()	do {	s_clear (&argument);	} while (0)


    TOPLOOP_SETJMP();

    CHECK_NO_ARGUMENTS();

    xprintf("\n%s\n", txts.Command_name);
    xprintf("-----------------------------------------------\n");
    index = 0;
    while (s_strcmp(cmd_list[index].name, EMPTY_STRING) != 0)
    {
	if (cmd_list[index].priority > 0)
	{
	    s_xputs(cmd_list[index].name);
	    newline();
	    index++;
	}
    }

    RETURN (OK);

#undef CLEAN_UP
}




/*  Show a list of unread news.  */
Export  Success
cmd_list_news (String   argument)
{
#define FUNCTION "cmd_list_news()"

/*    Success		  retval;*/
    Kom_err		  error;
    Conf_no_list	  list = EMPTY_CONF_NO_LIST;
    Conference		* conf = NULL;
    Membership	          ship = EMPTY_MEMBERSHIP;
    int	  		  sum = 0;	/* May be clobbered by longjmp */
    Membership_list	  ship_list = EMPTY_MEMBERSHIP_LIST;
    int			  pos;

#define CLEAN_UP()	do {	release_conf_no_list (&list);	\
				release_membership_list (&ship_list); \
				release_conf (conf); \
			} while (0)

    TOPLOOP_SETJMP();

    CHECK_NO_ARGUMENTS();


/* This should be done better. Perhaps with a function together with 
   cmd_goto_next_conf */
    if (kom_get_unread_confs (cpn, &list) == FAILURE) /* For speed. */
    {
	switch (kom_errno)
	{
	case  KOM_OUT_OF_MEMORY:
	    fatal1 (CLIENT_OUT_OF_MEMORY, "get_conf_no_list() failed");
	    break;

	case  KOM_NO_CONNECT:
	    fatal1 (CLIENT_CONNECTION_LOST, "get_conf_no_list() failed");
	    break;

	default:
	    fatal1 (CLIENT_UNEXPECTED_SERVER_ERROR,
				"get_conf_no_list() failed");
	    break;
	}
    }

    
    /* 
     * list now contains a list of conferences with possibly unread 
     * texts.
     * And ship_list contains a membership.
     */
    /*
     * What we have to do is: 
     * run through all conferences in membership present in list and
     * calculate and print unread articles.
     */

    xprintf("\n\n");

    for (pos = first_membership(cpn, &ship);
	 pos != -1;
	 pos = next_membership(&ship))
    {
	if ( membership_in_conf_no_list(ship, list) )
	{
	    int unread;
	    
	    unread = conf_last(ship.conf_no, &error) - 
		(ship.last_text_read + ship.no_of_read);
	    if (unread > 0) {
		xprintf ("%s %d %s ",
			txts.You_have,
			unread,
			(unread == 1 ? txts.unread_in_conf_singular
			 	     : txts.unread_in_conf_plural ));
		print_conf_name(ship.conf_no);
		xprintf (".\n");
		sum += unread;
	    }
	    release_conf (conf);
	}
    }
    if ( sum > 0 ) {
	xprintf ("\n%s %d %s.\n",
		 txts.Sum_of_unread,
		 sum,
		 (sum == 1 
		  ? txts.unread_totally_singular
		  : txts.unread_totally_plural ));
    } else {
	xprintf ("%s.\n", txts.You_have_seen_all_news);
    }
    
    RETURN (OK);
#undef CLEAN_UP
#undef FUNCTION
}



static is_waiting = 0;

/* Catch a LysKOM is saving message */
static void
catch_saving(void)
{
    if (is_waiting)
    {
	xputchar ('%');
	xflush (kom_outstream);
    }
}


/*  Wait for a message to be written. */
Export  Success
cmd_wait_for_text (String    argument)
{
#define FUNCTION "cmd_wait_for_text()"

    Kom_err		  error;
    Success		  retval;
    Conf_no_list	  list = EMPTY_CONF_NO_LIST;
    Membership	        * ship;
    Conf_no		  r;
    Conference		* conf = NULL;	/* May be clobbered by longjmp */
    int			  sum = 0;	/* May be clobbered by longjmp */
    Membership_list	  ship_list = EMPTY_MEMBERSHIP_LIST;
    fd_set		  readfds;

#define CLEAN_UP()	do {	release_conf_no_list (&list);	\
				release_conf (conf); \
			} while (0)

    TOPLOOP_SETJMP();

    CHECK_NO_ARGUMENTS();

    newline();

    is_waiting = 1;

    register_saving(&catch_saving);

    do {
	struct timeval	  timeout;

	FD_ZERO (&readfds);
	FD_SET (0, &readfds);	/* Stdin */

	timeout.tv_sec = DEFAULT_TIME_INTERVAL_WHILE_WAITING;
	timeout.tv_usec = 0;

	if (select(1, &readfds, NULL, NULL, &timeout) == 0)
	{
	    xputchar ('>');
	    xflush (kom_outstream);
	}

	if ((retval = kom_get_unread_confs (cpn, &list)) != FAILURE
	    && list.no_of_confs)
	    retval = kom_get_membership (cpn, 0, USHRT_MAX,
					 TRUE, &ship_list);
	if (retval == FAILURE)
	{
	    switch (kom_errno)
		{
		case  KOM_OUT_OF_MEMORY:
		    fatal1 (CLIENT_OUT_OF_MEMORY, "get_conf_no_list() failed");
		    break;

		case  KOM_NO_CONNECT:
		    fatal1 (CLIENT_SERVER_ERROR, "get_conf_no_list() failed");
		    break;

		default:
		    fatal1 (CLIENT_UNEXPECTED_SERVER_ERROR,
			    "get_conf_no_list() failed");
		    break;
		}
	}

    
	/* 
	 * list now contains a list of conferences with possibly unread 
	 * texts.
	 * And ship_list contains a membership.
	 */
	/*
	 * What we have to do is: 
	 * run through all conferences in membership present in list and
	 * calculate the sum of unread articles.
	 */

	if (list.no_of_confs) {
	    for (r = 0; r < ship_list.no_of_confs; r++)
	    {
		ship = &(ship_list.confs[r]);

		if ( membership_in_conf_no_list(*ship, list) )
		    {
			int unread;

			conf = conf_stat (ship->conf_no, &error);
			unread = conf->first_local_no + conf->no_of_texts - 1 - 
			    (ship->last_text_read + ship->no_of_read);
			if (unread > 0) {
			    sum += unread;
			}
			release_conf (conf);
		    }
	    }
	} else
	    sum = 0;

	release_membership_list(&ship_list);
	handle_all_asyncs();
    } while (!FD_ISSET( 0, &readfds ) && sum == 0);

    is_waiting = 0;

    if (sum != 0)
	xputchar ('\007');
    
    newline();

    RETURN (OK);
#undef CLEAN_UP
#undef FUNCTION
}


/*  Show who is logged in to the server, and what they are doing.  */
Export  Success
cmd_show_who_is_on (String   argument)
{
#define FUNCTION "cmd_show_who_is_on()"

    Success		  retval;
    Who_info_list	  who		= EMPTY_WHO_INFO_LIST;
    int			  i;
    int			  no_confs;

#define CLEAN_UP()	do {				\
	s_clear (&argument);				\
	release_who_info_list(&who);			} while (0)


    TOPLOOP_SETJMP();

    CHECK_NO_ARGUMENTS();

    newline();
    retval = kom_who_is_on (&who);
    if (retval == FAILURE)
    {
	switch (kom_errno)
	{
	case  KOM_OUT_OF_MEMORY:
	    fatal1 (CLIENT_OUT_OF_MEMORY, "who_is_on() failed");
	    break;

	case  KOM_NO_CONNECT:
	    fatal1 (CLIENT_SERVER_ERROR, "who_is_on() failed");
	    break;

	default:
	    fatal1 (CLIENT_UNEXPECTED_SERVER_ERROR, "who_is_on() failed");
	    break;
	}
    }

    /* So, let's print out the list */
    xprintf ("\n%5T%s%43T%s\n%5T%s%43T%s\n",
	     txts.Person, txts.Conference,
	     txts.Calling_in_from, txts.Is_doing);
    xprintf ("-------------------------------------------------------\n");

    no_confs = 0;

    for ( i = who.no_of_persons-1; i >= 0 ;  i-- )
    {
	xprintf ("%4d ", who.info[i].session_no);
	print_conf_name ((Conf_no) who.info[i].person);
	xprintf ("%43T");
	if (who.info[i].working_conference == 0)
	    xprintf ("%s.", txts.His_not_in_conf);
	else {
	    int r;

	    print_conf_name (who.info[i].working_conference);
	    /* Counting how many different confs */
	    for (r = 0; r < i; r++) 
	    {
		if (who.info[i].working_conference == who.info[r].working_conference) 
		{
		    no_confs --;
		    break;
		}
	    }
	    no_confs ++;
	}
	    
	xprintf ("%5T%S%43T%S\n",
		 who.info[i].username,
		 who.info[i].what_am_i_doing);
    }
    xprintf ("-------------------------------------------------------\n");
    xprintf (txts.Total_present_in_confs, 
	     who.no_of_persons,
	     no_confs);

    RETURN (OK);

#undef CLEAN_UP
#undef FUNCTION
}




/*  Start with a new name.  Does a login.  (The server automatically
 *  does a logout if already logged in.)
 */	
Export  Success
cmd_start_anew (String   argument)

{
#define FUNCTION "start_anew()"

    Pers_list		  possible_persons	= EMPTY_PERS_LIST;
    Pers_no		  new_user;
    String		  password		= EMPTY_STRING;
    String		  password2		= EMPTY_STRING;
    Success		  login_ok;

#define CLEAN_UP()	do {				\
	s_clear (&argument);				\
	s_clear (&password);				\
	s_clear (&password2);			} while (0)

    TOPLOOP_SETJMP();

    login_ok = FAILURE;
    while (login_ok == FAILURE)
    {
	/*  Was a name given as an argument, alternatively was the
	 *  previous attempt unsuccessful?			*/
	if (s_empty(argument))
	{
	    /* If not, ask for it */
	    xprintf("\n%s\n- ", txts.Whats_your_name);
	    read_line (&argument);
	    newline ();
	    printed_lines = 0;
	    current_column = 0;
	    argument = s_strip_trailing (argument, command_separators);
	    if (s_empty (argument))
		continue;
	}

	/* Find the number of the person */
	possible_persons = find_person_number (argument);

	/* Is there any such person? */
	if (possible_persons.no_of_persons != 1)
	{
	    /* No such person. Get password, try to create person
	     * and log in.					*/
	    xprintf ("\n%s.\n%s\n",
		    txts.Person_doesnt_exist,
		    txts.Check_spelling_or_press_return);
	    xprintf ("\n%s: ", txts.Give_personal_password);
	    read_password (&password);
	    if (s_empty (password))
	    {
		s_clear (&argument);
		continue;	/* Spelling mistake - try again */
	    }

	    xprintf ("\n%s: ", txts.Give_password_again);
	    read_password (&password2);
	    if (! s_usr_streq (password, password2, DEFAULT_COLLAT_TAB))
	    {
		xprintf ("%s.\n", txts.Password_doesnt_match);
		s_clear (&argument);
		continue;	/* Try again */
	    }

	    /* Try to create the person */
	    new_user = kom_create_person (argument, password);
	    if (new_user == 0)
	    {
		switch (kom_errno)
		{
		case  KOM_LOGIN:
		    xprintf ("%s.\n", txts.Cant_create_pers_unlogined);
		    break;

		case  KOM_BAD_NAME:
		    xprintf ("%s.\n", txts.Name_was_bad);
		    break;

		case  KOM_CONF_EXISTS:
		case  KOM_PERS_EXISTS: /* Someone was faster than us :-) */
		    xprintf ("%s.\n", txts.Name_occupied);
		    break;

		case  KOM_NO_CONNECT:
		    fatal1 (CLIENT_SERVER_ERROR, "create_person() failed");
		    break;

		default:
		    fatal2 (CLIENT_UNEXPECTED_SERVER_ERROR,
			    "create_person() failed: ", kom_errno);
		    break;
		}

		/* Signal unsuccessful login to next loop */
		s_clear (&argument);
		continue;	/* And give the user a new try */
	    }	/* If create_person() failed */
	}
	else
	{
	    /* Yes, person exists. Then get password and try to log in */
	    /* First tell the full name of the person */
	    new_user = possible_persons.persons[0];
	    newline();
	    print_conf_name ((Conf_no) new_user);
	    xprintf("\n%s: ", txts.Password);
	    read_password (&password);
	}

	/* Do the login. */
	login_ok = kom_login (new_user, password, 0);
	if (login_ok == FAILURE)
	{
	    switch (kom_errno) 
	    {
	    case KOM_PWD:	/* This is the "normal" error here */
		s_clear (&argument);
		break;
	    case KOM_CLIENT_IS_CRAZY: /* On old servers that cannot */
				/* handle the (new) login call we try */
				/* to login using the kom_login_old. */
		login_ok = kom_login_old (new_user, password);
		if (login_ok == OK)
		{		/* This did the trick. */
		    cache_change_person (new_user);
		    break;
		}
		/* FALLTHRU */
	    default:
		fatal1 (CLIENT_SERVER_ERROR,
		       "couldn't login\n");
		/*NOTREACHED*/
	    }
	}
	else
	    cache_change_person (new_user);
	
	zfree (possible_persons.persons);
    }	/* While login_ok == FAILURE */

    {
	String	  	  str;	/* May be clobbered by longjmp */
	unsigned char	  buf[100];

	sprintf(buf, txts.i_am_doing_startup, VERSION_ID);
	str.len = strlen(buf);
	str.string = buf;
	
	kom_change_what_i_am_doing(str);
    }

    RETURN (login_ok);
#undef CLEAN_UP
#undef FUNCTION
}



Export  Success
cmd_enable_user (String   argument)

{
    unsigned char	  enable_level;
    String_size		  error_check;
#define FUNCTION "cmd_enable_user()"

#define CLEAN_UP()	do {	s_clear (&argument);	} while (0)

    TOPLOOP_SETJMP();

    if (s_empty (argument))
    {
	xprintf ("\n%s - ", txts.What_enable_level);
	read_line (&argument);
	newline();
	argument = s_strip_trailing (argument, command_separators);
	if (s_empty (argument))
	{
	    RETURN (FAILURE);
	}
    }

    enable_level = s_strtol (argument, &error_check,
			     DEFAULT_NUMBER_BASE);
    if (error_check < s_strlen (argument))
    {
	/*  No, it wasn't a number.  */
	xprintf ("%s:  ", txts.Unknown_argument);
	s_puts (argument);
	newline();
	RETURN (FAILURE);
    }
    xprintf ("%d\n", enable_level);

    if (kom_enable (enable_level) != OK)
    {
	fatal1 (CLIENT_ERROR_TOO_boring, "kom_query_read_texts() failed");
    }

    RETURN (OK);

#undef CLEAN_UP
#undef FUNCTION
}




Export  Success
cmd_get_encouragement (String   argument)

{
#define CLEAN_UP()	do {	s_clear (&argument);	} while (0)

    TOPLOOP_SETJMP();

    CHECK_NO_ARGUMENTS();

    xprintf(txts.encouragement);
    RETURN (OK);

#undef CLEAN_UP
}



extern time_t time(time_t *);
/*
 *  Print the current time
 */
Export  Success
cmd_show_current_time (String	argument)
{
    time_t			  cur_time;

#define CLEAN_UP()	do {	s_clear (&argument);	} while (0)

    TOPLOOP_SETJMP();

    CHECK_NO_ARGUMENTS();

    cur_time = time (NULL);
    print_time (localtime(&cur_time));
    newline();

    RETURN (OK);

#undef CLEAN_UP
}



Export  Success
cmd_show_moon_phase (String	argument)

{
    static char		* phase_names[4] = { "NM", "FQ", "FM", "LQ" };
    time_t		  cur_time;
    struct pom		  phase;

#define CLEAN_UP()	do { } while (0)

    TOPLOOP_SETJMP();

    CHECK_NO_ARGUMENTS();

    newline();
    cur_time = time (NULL);
    phase = phase_of_the_moon (gmtime (&cur_time));

    /* BUG! */
    xputs (phase_names[phase.quarter]);
    if (phase.days + phase.hours + phase.minutes + phase.seconds >= 0)
	xputs (" + ");
    else
    {
	xputs (" - ");
	phase.days = -phase.days;
	phase.hours = -phase.hours;
	phase.minutes = -phase.minutes;
	phase.seconds = -phase.seconds;
    }
    xprintf ("%dd %dh %dm %ds\n", phase.days, phase.hours,
	     phase.minutes, phase.seconds);

    RETURN (OK);
#undef CLEAN_UP
}   /* END: cmd_show_moon_phase() */



Export  Success
cmd_answer_never_existed (String	argument)
{
#define CLEAN_UP()	do {	s_clear (&argument);	} while (0)

    TOPLOOP_SETJMP();

    s_puts (argument);
    xprintf ("\n\n%s.\n%s\n",
	    txts.Answer_never_existed,
	    txts.Use_QZ_KOM_instead);
    RETURN (OK);
#undef CLEAN_UP
}   /* END: cmd_answer_never_existed() */



/*
 *  Spawn an exec.  VERY dirty written.
 */
Export  Success
cmd_spawn_exec (String	  argument)
{
    int			  retval;
    char		* err_msg;

    extern char		* sys_errlist[];
    extern int		  fork (void);
    extern int		  execlp (char *, char *,...);
    extern int		  errno;

#define CLEAN_UP()	do { } while (0)

    TOPLOOP_SETJMP();

    CHECK_NO_ARGUMENTS();
    newline();

    restore_terminal (stdin, stdout);

    if ((retval = fork()) == 0)
    {
	if (execlp ("egzek", "egzek", (char *)NULL) == -1)
	{
	    err_msg = sys_errlist[errno];
	    xprintf ("%s.\n%s: %s\n",
		     txts.Cant_start_exec,
		     txts.OS_gives_errmsg,
		     err_msg);
	    exit (1);
	}
    }
    else if (retval > 0)
    {
	(void) wait (NULL);
    }
    else
    {
	err_msg = sys_errlist[errno];
	init_terminal (stdin, stdout);
	xprintf ("%s.\n%s: %s\n",
		 txts.Cant_start_exec,
		 txts.OS_gives_errmsg,
		 err_msg);
	RETURN (FAILURE);
    }

    init_terminal (stdin, stdout);
    RETURN (OK);
#undef CLEAN_UP
}   /* END: cmd_spawn_exec() */



/*
 * Send a personal message.
 */
Export	Success
cmd_send_personal (String	argument)
{
#define FUNCTION "cmd_send_personal()"
    Micro_conf		  reciever;
    Conf_no		  recno;
    Kom_err		  error;
    Who_info_list	  who_list		= EMPTY_WHO_INFO_LIST;
    String		  message		= EMPTY_STRING;
    int			  r;
    String		  name 			= EMPTY_STRING;

#define CLEAN_UP()	do {				\
	s_clear (&argument);				\
	s_clear (&message);				\
        zfree(name.string);				\
	release_who_info_list( &who_list );  } while (0)

    TOPLOOP_SETJMP();

    reciever = ask_for_conf(& argument,
			    txts.Who_to_send_message_to);
    if (reciever.conf_no == 0)
    {
	RETURN (FAILURE);
    }

    if (!reciever.type.letter_box)
    {
	name = conf_name(reciever.conf_no,&error);
	xprintf(txts.Not_a_person,&name);
	RETURN (FAILURE);
    }

    if (kom_who_is_on ( &who_list ) == FAILURE) {
	fatal1(CLIENT_ERROR_TOO_boring, 
	       "Cannot find list of logged in persons");
    }

    recno = 0;
    for ( r = 0; r < who_list.no_of_persons; r++ )
	if (who_list.info[r].person == reciever.conf_no)
	    recno = reciever.conf_no;

    if (recno == 0) {
	name = conf_name(reciever.conf_no,&error);
	xprintf(txts.Not_currently_logged_in,name);
	RETURN (FAILURE);
    }

    xprintf(txts.Message_to_send);
    read_line(&message);
    newline();

    if (s_strlen(message) > 0)
    {
	if (kom_send_message(recno, message) != FAILURE)
	{
	    name = conf_name(recno, &error);
	    xprintf(txts.Message_sent_to, name);
	}
	else
	{
	    xprintf(txts.Message_not_sent);
	}
    }
    else
    {
	xprintf(txts.Tant_pis);
    }
	

    RETURN (OK);

#undef CLEAN_UP
#undef FUNCTION
}    


/*
 * Send a general message.
 */
Export	Success
cmd_send_alarm (String	argument)
{
    String		  message		= EMPTY_STRING;

#define CLEAN_UP()	do {				\
	s_clear (&argument);				\
	s_clear (&message);	} while (0)

    TOPLOOP_SETJMP();

    CHECK_NO_ARGUMENTS();

    newline();

    xprintf(txts.Message_to_send);
    read_line(&message);
    newline();

    if (s_strlen(message) > 0) 
    {
	if (kom_send_message(0, message) != FAILURE)
	{
	    xprintf(txts.Message_sent_to_all);
	}
	else
	{
	    xprintf(txts.Message_not_sent);
	}
    }
    else
    {
	xprintf(txts.Tant_pis);
    }

    RETURN (OK);

#undef CLEAN_UP
}    



Export	void
show_directed_message(Conf_no	to, 
		      Conf_no	from,
		      String	message)
{
    Kom_err	error;
    String 	name = conf_name(from, &error);

#define CLEAN_UP()	do { zfree(name.string); } while (0)

    TOPLOOP_SETJMP();

    newline();
    if (to == 0)
	xprintf(txts.global_message, name, message);
    else
        xprintf(txts.personal_message, name, message);

    RETURNVOID;
#undef CLEAN_UP
}
	

/*  ==================================================================  */
/*		Internal functions for 'functions-*.c'			*/



/*
 *  Ask the user to answer yes, no or maybe. Any illegal answer is
 *  taken as no.
 */
/* PORT: The words Yes, No and Maybe must be unique in the first
 * 	 character to be recognized correctly!
 */
Export  Bool
yes_or_no_p (void)
{
#define FUNCTION "yes_or_no_p()"

    String		  input		= EMPTY_STRING;
    String		  first_word	= EMPTY_STRING;
    Bool		  result;
    String_size		  start_pos;
    int			  i;
    time_t		  cur_time;
    struct pom		  phase;

    #define		  limit		3
    #define		  wait_time	1	/* 1 s */


#define CLEAN_UP()	do {	s_clear (&input);	} while (0)

    TOPLOOP_SETJMP();

    xprintf ("(%s, %s, %s) - ",
	    txts.yes, txts.no, txts.maybe);
    if (read_line (&input) == FAILURE)
    {
	/* Memory seems to be a scarce commodity */
	fatal1 (CLIENT_OUT_OF_MEMORY, "read_line() failed");
    }
    newline();

    remove_parenthesis (&input, command_separators.string[0]);
    input = s_strip_trailing (input, command_separators);
    start_pos = 0;
    first_word = s_strtok (input, &start_pos, command_separators);

    if ((start_pos < s_strlen (input)) || (s_empty (first_word)))
    {
	/* Either more than one word, or no words at all */
	xprintf ("%s.\n", txts.Answer_yes_no_maybe);
	result = FALSE;
    }
    else if (s_usr_strhead (first_word,
		       s_fcrea_str (txts.yes),
		       DEFAULT_COLLAT_TAB))
	result = TRUE;
    else if (s_usr_strhead (first_word,
			    s_fcrea_str (txts.no),
			    DEFAULT_COLLAT_TAB))
	result = FALSE;
    else if (s_usr_strhead (first_word,
			    s_fcrea_str (txts.maybe),
			    DEFAULT_COLLAT_TAB))
    {
	xprintf ("%s", txts.Checking_phase_of_moon);
	for ( i = 0 ;  i < limit ;  i++)
	{
	    extern void sleep(int);

	    xputchar ('.');
	    xflush (kom_outstream);
	    sleep (wait_time);
	}
	
	cur_time = time (NULL);
	phase = phase_of_the_moon (gmtime (&cur_time));
	if ((result = (Bool) (phase.seconds | 1)))
	    xprintf ("  %s.\n", txts.yes);
	else
	    xprintf ("  %s.\n", txts.no);
    }
    else if (2[input.string]+input.string[3]*(UCHAR_MAX+1)==
	     (UCHAR_MAX+2)*RTFS&&input.len&5==5&&*input.string
	     ==MIHALCOC&&(input.len&0x2a)[input.string]*(UCHAR_MAX+1)
	     +input.string[input.len&011477]==(UCHAR_MAX+2)*FFGZ)
    {
	xprintf ("%s.\n", txts.Nothing_happens);
	result = FALSE;
    }
    else
    {
	xprintf ("%s.\n", txts.Answer_yes_no_maybe);
	result = FALSE;
    }

    RETURN (result);

#undef CLEAN_UP
#undef FUNCTION
}
