/*
 *  functions-edit.c
 *		Functions for the commands that edit a text being written.
 *
 *
 *  Copyright (C) 1990	Lysator Computer Club,
 *			Linkoping University,  Sweden
 *
 *  Everyone is granted permission to copy, modify and redistribute
 *  this code, provided the people they give it to can.
 *
 *
 *  Author:	Thomas Bellman
 *		Lysator Computer Club
 *		Linkoping University
 *		Sweden
 *
 *  email:	Bellman@Lysator.LiU.SE
 *
 *
 *  Any opinions expressed in this code are the author's PERSONAL opinions,
 *  and does NOT, repeat NOT, represent any official standpoint of Lysator,
 *  even if so stated.
 */

#include <stdio.h>
/* #include <time.h> included from kom-types.h */
#if STDC_HEADERS || HAVE_STRING_H
#include <string.h>
#else
#include <strings.h>
#endif

#ifdef __vax__
#define __SIZE_T included from string.h
#endif
#include <kom-types.h>	/* Before zmalloc.h */
#include <zmalloc.h>
#include <s-string.h>
#include <s-collat-tabs.h>

#include <config.h>

#include <kom-errno.h>
#include <services.h>

#include "conf.h"

/* Must be included before "read-line.h" on a vax because it includes <sgtty.h>
 * and that does not work after inclusion of <sys/ioctl.h> because of
 * stupid include-file construction.
 */
#include "xoutput.h"

#include "read-line.h"
#include "commands.h"
#include "parser.h"
#include "texts.h"
#include "edit.h"
#include "error.h"
#include "cache.h"
#include "quit.h"

#include "internal.h"
#include "cmds.h"


/* This is to emphasize that a function is exported. */
#define Export


/*  ==================================================================  */
/*			    Exported functions				*/


Export  Success
cmd_mark_permanently (String	  argument)

{
    Success			  retval;
    Text_no			  text_to_mark;
    String_size			  error_check;

#define CLEAN_UP()	do	{	s_clear (&argument);	}while (0)

    TOPLOOP_SETJMP();

    if (s_empty (argument))
    {
	text_to_mark = reading_state.last_viewed;
    }
    else
    {
	text_to_mark = s_strtol (argument, &error_check,
				 DEFAULT_NUMBER_BASE);
	if (error_check < s_strlen (argument))
	{
	    /*  No, it wasn't a number.  */
	    xprintf ("%s:  ", txts.Unknown_argument);
	    s_puts (argument);
	    newline();
	    RETURN (FAILURE);
	}
    }
    xprintf ("%d", text_to_mark);
    newline();

    retval = kom_mark_text (text_to_mark, 255);		/* 255 is a BUG! */
    if (retval != OK)
    {
	xprintf ("%s.\n", txts.Cant_mark_text);
	RETURN (FAILURE);
    }

    RETURN (OK);

#undef CLEAN_UP
}   /* END: cmd_mark_permanently() */



Export	Success
cmd_delete_text (String		  argument)

{
    Text_no			  text_to_delete;
    String_size			  error_check;

#define CLEAN_UP()	do	{ 	s_clear (&argument);	} while (0)

    TOPLOOP_SETJMP();

    if (s_empty (argument))
    {	
	xprintf(txts.Give_the_textno_to_delete);
	RETURN (FAILURE);
    }
    else
    {
	text_to_delete = s_strtol (argument, &error_check,
				   DEFAULT_NUMBER_BASE);
	if (error_check < s_strlen (argument))
	{
	    /*  No, it wasn't a number.  */
	    xprintf ("%s:  ", txts.Unknown_argument);
	    s_puts (argument);
	    newline();
	    RETURN (FAILURE);
	}
	xprintf ("%d", text_to_delete);
    }

    newline();
    xprintf (txts.Really_sure_delete_text, text_to_delete);

    if (yes_or_no_p())
    {
	if (kom_delete_text( text_to_delete ) != FAILURE)
	{
	    RETURN (OK);
	}
	else
	{
	    xprintf(txts.Delete_text_failed, text_to_delete);
	    RETURN (FAILURE);
	}
    }
    else
    {
	xprintf (txts.Tant_pis);
    }
    RETURN (OK);
}
