/*
 *  copy.c
 *	Functions to copy and free the data structures used by LysKOM.
 *
 *
 *  Copyright (C) 1990	Lysator Computer Club,
 *			Linkoping University,  Sweden
 *
 *  Everyone is granted permission to copy, modify and redistribute
 *  this code, provided the people they give it to can.
 *
 *
 *  Author:	Thomas Bellman
 *		Lysator Computer Club
 *		Linkoping University
 *		Sweden
 *
 *  email:	Bellman@Lysator.LiU.SE
 *
 *  Any opinions expressed in this code are the author's PERSONAL opinions,
 *  and does NOT, repeat NOT, represent any official standpoint of Lysator,
 *  even if so stated.
 */


#include <compiler.h>

#if STDC_HEADERS || HAVE_STRING_H
#include <string.h>
#else
#include <strings.h>
#endif

#include <kom-types.h>
#ifdef __vax__
#define __SIZE_T included from string.h
#endif
#include <zmalloc.h>
#include <s-string.h>

#include "copy.h"

#define Export



Export  Person *
release_person (Person	* pers)
{
    if (pers == NULL)
	return  pers;
    (void) s_release (&(pers->username));
    return  pers;
}



Export  Conference *
release_conf (Conference	* conf)
{
    if (conf == NULL)
	return  conf;
    (void) s_release (&(conf->name));
    return  conf;
}



Export  Misc_info *
release_misc_list (Misc_info	* list,
		   int		  length)
{
    if (list == NULL)
	return  list;
    while (--length >= 0)
	;
    return  list;
}



Export  Text_stat *
release_text_stat (Text_stat	* text)
{
    if (text == NULL)
	return  text;
    zfree (release_misc_list (text->misc_items, text->no_of_misc));
    return  text;
}



Export  Membership *
release_membership (Membership	* memb)
{
    if (memb == NULL)
	return  memb;
    zfree (memb->read_texts);
    return  memb;
}


Export  Mark_list *
release_mark_list (Mark_list	* marks)
{
    unsigned short	  i;

    if (marks == NULL)
	return  marks;
    for (i = 0 ;  i < marks->no_of_marks ;  i++)
	;
    zfree (marks->marks);
    return  marks;
}


Export  Conf_no_list *
release_conf_no_list (Conf_no_list * list)
{
    if (list == NULL)
	return  list;
    zfree (list->conf_nos);
    return list;
}
  

Export  Member_list *
release_member_list (Member_list	* membs)
{
    if (membs == NULL)
	return  membs;
    zfree (membs->members);
    return  membs;
}



Export  Text_list *
release_text_list (Text_list	* texts)
{
    if (texts == NULL)
	return  texts;
    zfree (texts->texts);
    return  texts;
}



Export  Conf_list *
release_conf_list (Conf_list	* confs)
{
    if (confs == NULL)
	return  confs;
    zfree (confs->confs);
    return  confs;
}



Export  Pers_list *
release_pers_list (Pers_list	* persons)
{
    if (persons == NULL)
	return  persons;
    zfree (persons->persons);
    return  persons;
}



Export  Membership_list *
release_membership_list (Membership_list	* membships)
{
    long	  i;

    if (membships == NULL)
	return  membships;
    for ( i = membships->no_of_confs-1 ;  i > 0 ;  i-- )
    {
	release_membership (&membships->confs[i]);
    }
    zfree (membships->confs);
    return  membships;
}



Export  Info *
release_info (Info	* info)
{
    return  info;
}



Export  Who_info *
release_who_info (Who_info	* info)
{
    if (info == NULL)
	return  info;
    (void) s_release (& (info->what_am_i_doing));
    (void) s_release (& (info->username));
    return  info;
}



Export  Who_info_list *
release_who_info_list (Who_info_list	* who)
{
    long	  i;

    if (who == NULL)
	return  NULL;
    for ( i = who->no_of_persons - 1 ;  i >= 0 ;  i--)
    {
	release_who_info (& who->info[i]);
    }
    zfree (who->info);
    return  who;
}



Export  Person *
mark_person (Person	* pers)
{
    (void) s_mark (&pers->username);
    return  pers;
}



Export  Conference *
mark_conf (Conference	* conf)
{
    (void) s_mark (&conf->name);
    return  conf;
}



Export  Misc_info *
mark_misc_list (Misc_info	* list,
		int		  length)
{
    while (--length >= 0)
	/* Do nothing on list[length] */    ;
    return  list;
}



Export  Text_stat *
mark_text_stat (Text_stat	* text)
{
    mark_misc_list (text->misc_items, text->no_of_misc);
    return  text;
}



Export  Membership *
mark_membership (Membership	* memb)
{
    zuse (memb->read_texts);
    return  memb;
}



Export  Mark_list *
mark_mark_list (Mark_list	* marks)
{
    unsigned short	  i;

    for (i = 0 ;  i < marks->no_of_marks ;  i++)
	/* Do nothing */    ;
    zuse (marks->marks);
    return  marks;
}



Export  Member_list *
mark_member_list (Member_list	* membs)
{
    zuse (membs->members);
    return  membs;
}



Export  Text_list *
mark_text_list (Text_list	* texts)
{
    zuse (texts->texts);
    return  texts;
}



Export  Conf_list *
mark_conf_list (Conf_list	* confs)
{
    zuse (confs->confs);
    return  confs;
}



Export  Pers_list *
mark_pers_list (Pers_list	* persons)
{
    zuse (persons->persons);
    return  persons;
}



Export  Membership_list *
mark_membership_list (Membership_list	* membships)
{
    unsigned short	  i;

    for (i = 0 ;  i < membships->no_of_confs ;  i++)
	zuse (membships->confs[i].read_texts);
    zuse (membships->confs);
    return  membships;
}



Export  Info *
mark_info (Info	    * info)
{
    return  info;
}
