/*
 *  client/main.c  --  LysKOM client, main program
 *
 *
 *  Copyright (C) 1990  Lysator Computer Club
 *
 *  Everyone is granted permission to copy, modify and redistribute
 *  this code, provided the people they give it to can.
 *
 *
 *  Author:	Thomas Bellman
 *		Lysator Computer Club
 *		Linkoping University
 *		Sweden
 *
 *  E-mail:	Bellman@Lysator.LiU.SE
 */


#include <stdio.h>
#if HAVE_STDLIB_H
#include <stdlib.h>
#endif

#ifdef __vax__
#define __SIZE_T included from stdlib.h
#define __WCHAR_T included from stdlib.h
#endif
#include <kom-types.h>
#include <zmalloc.h>
#include <s-string.h>

#include <services.h>

#include "version.h"

/* Must be included before "read-line.h" on a vax because it includes <sgtty.h>
 * and that does not work after inclusion of <sys/ioctl.h> because of
 * stupid include-file construction.
 */
#include "xoutput.h"

#include "error.h"
#include "read-line.h"
#include "texts.h"
#include "commands.h"
#include "cmds.h"
#include "cache.h"
#include "edit.h"

#include "defaults.h"

char		* server_name;
Info		  serv_info;
char		* client_name;

int
main (int	   argc,
      char	** argv)

{
#define FUNCTION "main()"

    s_set_storage_management (zmalloc, zrealloc, zfree);
    kom_set_storage_management (zmalloc, zrealloc, zfree);
    init_messages();	/* Initialize the 'txts' struct */
    init_cmd_list();	/* Initialize the list of commands */
    init_terminal (stdin, stdout);
    init_cache();

    client_name = argv[0];
    if (!client_name)
	client_name = "(unknown)";

    server_name = DEFAULT_SERVER;

    /* Aronsson was here: 7 bit character size conversion for terminals */
    /* For now, argument parsing is very primitive. */
    /* Perhaps we should consider GNU's getopt. */
    while (argc > 1)
    {
	if (0 == strcmp (argv[1], "--swascii"))
	    xo_cs7 = 1;
	else if (0 == strcmp (argv[1], "--tomradavslutar"))
	    tomrad_avslutar = 1;
	else if (argv[1][0] != '-')
	{
	    server_name = argv[1];
	} else {
	    xprintf ("Usage: [--swascii] [--tomradavslutar] [server]\n");
	    exit(1);
	}
	
	argc --;
	argv ++;
    }
    /* Here argv and argc are irreparably destroyed. */


    if (kom_open_connection(server_name) == FAILURE)
    {
	xprintf (txts.no_contact_with_server, server_name);
	exit(1);
    }

    /*  OK, connection established  */
    kom_get_info (&serv_info);	/* BUG! Retval not checked */

    /*  Greet user  */
    xprintf (txts.greet_user /* , serv_info.server_id  Vad {r detta? */,
	     serv_info.version,
	     VERSION_ID);

    /*  Log in to server  */
    cmd_start_anew (EMPTY_STRING);
    
    /*  And run the command loop  */
    command_loop ();

    /* Restore the terminal to the old settings */
    restore_terminal (stdin, stdout);

    return OK;	/*  LysKOM can *never* be wrong! */
#undef FUNCTION
}

